@@ -111,6 +111,15 @@ class Loader:
         handler. When disabled, the exceptions are not raised
         and the condition is considered False.
 
+    uniondebugconflict: Disabled by default
+        When enabled, all the possible types for the unions
+        are evaluated instead of stopping at the first that works.
+        If more than one type in the union works, an error is raised
+        because the types are conflicting and the union might return
+        different types with the same input value.
+        This option makes the loading slower and is only to be used when
+        debugging issues.
+
     mangle_key: Defaults to 'name'
         Specifies which key is used into the metadata dictionaries
         to perform name-mangling.
@@ -193,6 +202,9 @@ class Loader:
         # Which key is used in metadata to perform name mangling
         self.mangle_key = 'name'
 
+        # Fail if multiple types work within the union
+        self.uniondebugconflict = False
+
         # Objects loaded from a string
         self.strconstructed = {
             Path,
@@ -293,7 +305,7 @@ class Loader:
                 value = {k: v for k,v in value._get_kwargs()}
 
         try:
-            return cast(T, func(self, value, type_))
+            return func(self, value, type_)
         except Exception as e:
             assert isinstance(e, TypedloadException)
             e.trace.insert(0, TraceItem(value, type_, annotation))
@@ -467,12 +479,12 @@ def _namedtupleload(l: Loader, value: Dict[str, Any], type_) -> Any:
         type_hints = type_.__annotations__
     else:
         #dataclass
-        import dataclasses
+        from dataclasses import _MISSING_TYPE as DT_MISSING_TYPE
         fields = set(type_.__dataclass_fields__.keys())
         optional_fields = {k for k,v in type_.__dataclass_fields__.items() if
                            v.init == False or
-                           not isinstance(v.default, dataclasses._MISSING_TYPE) or
-                           not isinstance(v.default_factory, dataclasses._MISSING_TYPE)}
+                           not isinstance(v.default, DT_MISSING_TYPE) or
+                           not isinstance(v.default_factory, DT_MISSING_TYPE)}
         type_hints = {k: v.type for k,v in type_.__dataclass_fields__.items()}
 
         #Name mangling
@@ -573,17 +585,37 @@ def _unionload(l: Loader, value, type_) -> Any:
             sorted_args.append(t)
 
     # Try all types
+    loaded_count = 0
+    r = None
     for t in sorted_args:
         try:
-            return l.load(value, t, annotation=Annotation(AnnotationType.UNION, t))
+            r = l.load(value, t, annotation=Annotation(AnnotationType.UNION, t))
+            loaded_count += 1
+            if not l.uniondebugconflict:
+                # Do not try more if we are not debugging
+                break
         except Exception as e:
             exceptions.append(e)
-    raise TypedloadValueError(
-        'Value of %s could not be loaded into %s' % (tname(value_type), tname(type_)),
-            value=value,
-            type_=type_,
-            exceptions=exceptions
-    )
+
+    if loaded_count == 1:
+        # Loaded only once, all good
+        return r
+    elif loaded_count == 0:
+        # Could not be loaded
+        raise TypedloadValueError(
+            'Value of %s could not be loaded into %s' % (tname(value_type), tname(type_)),
+                value=value,
+                type_=type_,
+                exceptions=exceptions
+        )
+    else:
+        # Loaded more than once, conflict
+        raise TypedloadTypeError(
+            'Value of %s could be loaded into %s %d times' % (tname(value_type), tname(type_), loaded_count),
+                value=value,
+                type_=type_,
+                exceptions=exceptions
+        )
 
 
 def _enumload(l: Loader, value, type_) -> Enum:
@@ -676,7 +708,7 @@ def _attrload(l, value, type_):
 
 def _strconstructload(l: Loader, value, type_):
     """
-    Loader for all the types in the stdlib ipaddress module.
+    Loader for all the types taking a string as single constructor parameter
     """
     try:
         return type_(value)
